@@ -1,4 +1,5 @@
 import pendulum
+
 from dagster import check
 from dagster.core.execution.backfill import (
     BulkActionStatus,
@@ -42,7 +43,18 @@ def create_and_launch_partition_backfill(graphene_info, backfill_params):
 
     external_partition_set = next(iter(matches))
 
-    partition_names = backfill_params.get("partitionNames")
+    if backfill_params.get("allPartitions"):
+        result = graphene_info.context.get_external_partition_names(
+            repository.handle, external_partition_set.name
+        )
+        partition_names = result.partition_names
+    elif backfill_params.get("partitionNames"):
+        partition_names = backfill_params.get("partitionNames")
+    else:
+        raise Exception(
+            'Backfill requested without specifying either "allPartitions" or "partitionNames" '
+            "arguments"
+        )
 
     backfill_id = make_new_backfill_id()
     backfill = PartitionBackfill(
@@ -67,7 +79,11 @@ def create_and_launch_partition_backfill(graphene_info, backfill_params):
             submitted_run_ids.extend(
                 run_id
                 for run_id in submit_backfill_runs(
-                    graphene_info.context.instance, location, backfill, partition_names=chunk
+                    graphene_info.context.instance,
+                    workspace=graphene_info.context,
+                    repo_location=location,
+                    backfill_job=backfill,
+                    partition_names=chunk,
                 )
                 if run_id != None
             )
