@@ -1,5 +1,7 @@
 from typing import TYPE_CHECKING, Dict, Mapping
 
+from dagster_graphql.implementation.loader import CrossRepoAssetDependedByLoader
+
 from dagster import AssetKey, DagsterEventType, EventRecordsFilter, check, seven
 from dagster.core.events import ASSET_EVENTS
 
@@ -64,6 +66,8 @@ def get_asset_nodes_by_asset_key(graphene_info) -> Mapping[AssetKey, "GrapheneAs
 
     from ..schema.asset_graph import GrapheneAssetNode
 
+    depended_by_loader = CrossRepoAssetDependedByLoader(context=graphene_info.context)
+
     asset_nodes_by_asset_key: Dict[AssetKey, GrapheneAssetNode] = {}
     for location in graphene_info.context.repository_locations:
         for repository in location.get_repositories().values():
@@ -71,7 +75,10 @@ def get_asset_nodes_by_asset_key(graphene_info) -> Mapping[AssetKey, "GrapheneAs
                 preexisting_node = asset_nodes_by_asset_key.get(external_asset_node.asset_key)
                 if preexisting_node is None or preexisting_node.external_asset_node.op_name is None:
                     asset_nodes_by_asset_key[external_asset_node.asset_key] = GrapheneAssetNode(
-                        location, repository, external_asset_node
+                        location,
+                        repository,
+                        external_asset_node,
+                        depended_by_loader=depended_by_loader,
                     )
 
     return asset_nodes_by_asset_key
