@@ -1,4 +1,5 @@
 import graphene
+
 from dagster import check
 from dagster.core.definitions.events import AssetKey
 from dagster.utils.error import SerializableErrorInfo
@@ -97,6 +98,29 @@ class GraphenePipelineNotFoundError(graphene.ObjectType):
         )
 
 
+class GrapheneGraphNotFoundError(graphene.ObjectType):
+    class Meta:
+        interfaces = (GrapheneError,)
+        name = "GraphNotFoundError"
+
+    graph_name = graphene.NonNull(graphene.String)
+    repository_name = graphene.NonNull(graphene.String)
+    repository_location_name = graphene.NonNull(graphene.String)
+
+    def __init__(self, selector):
+        from ..implementation.utils import GraphSelector
+
+        super().__init__()
+        check.inst_param(selector, "selector", GraphSelector)
+        self.graph_name = selector.graph_name
+        self.repository_name = selector.repository_name
+        self.repository_location_name = selector.location_name
+        self.message = (
+            "Could not find Graph "
+            f"{selector.location_name}.{selector.repository_name}.{selector.graph_name}"
+        )
+
+
 class GraphenePipelineRunNotFoundError(graphene.Interface):
     class Meta:
         interfaces = (GrapheneError,)
@@ -180,6 +204,20 @@ class GrapheneModeNotFoundError(graphene.ObjectType):
         self.message = f"Mode {mode} not found in pipeline {selector.pipeline_name}."
 
 
+class GrapheneNoModeProvidedError(graphene.ObjectType):
+    class Meta:
+        interfaces = (GrapheneError,)
+        name = "NoModeProvidedError"
+
+    pipeline_name = graphene.NonNull(graphene.String)
+
+    def __init__(self, pipeline_name, mode_list):
+        super().__init__()
+        mode_list = check.list_param(mode_list, "mode_list", of_type=str)
+        pipeline_name = check.str_param(pipeline_name, "pipeline_name")
+        self.message = f"No mode provided for pipeline '{pipeline_name}', which has multiple modes. Available modes: {mode_list}"
+
+
 class GrapheneInvalidStepError(graphene.ObjectType):
     invalid_step_key = graphene.NonNull(graphene.String)
 
@@ -213,6 +251,7 @@ class GrapheneRunConflict(graphene.ObjectType):
 create_execution_params_error_types = (
     GraphenePresetNotFoundError,
     GrapheneConflictingExecutionParamsError,
+    GrapheneNoModeProvidedError,
 )
 
 
@@ -310,6 +349,7 @@ types = [
     GrapheneInvalidPipelineRunsFilterError,
     GrapheneInvalidStepError,
     GrapheneModeNotFoundError,
+    GrapheneNoModeProvidedError,
     GraphenePartitionSetNotFoundError,
     GraphenePipelineNotFoundError,
     GraphenePipelineRunConflict,
