@@ -236,6 +236,7 @@ class GrapheneRun(graphene.ObjectType):
     rootRunId = graphene.Field(graphene.String)
     parentRunId = graphene.Field(graphene.String)
     canTerminate = graphene.NonNull(graphene.Boolean)
+    assetMaterializations = non_null_list(GrapheneMaterializationEvent)
     assets = non_null_list(GrapheneAsset)
     events = graphene.Field(
         non_null_list(GrapheneDagsterRunEvent),
@@ -357,13 +358,17 @@ class GrapheneRun(graphene.ObjectType):
     def resolve_assets(self, graphene_info):
         return get_assets_for_run_id(graphene_info, self.run_id)
 
+    def resolve_assetMaterializations(self, graphene_info):
+        # convenience field added for users querying directly via GraphQL
+        return [
+            GrapheneMaterializationEvent(event=event)
+            for event in graphene_info.context.instance.all_logs(
+                self.run_id, of_type=DagsterEventType.ASSET_MATERIALIZATION
+            )
+        ]
+
     def resolve_events(self, graphene_info, after=-1):
         events = graphene_info.context.instance.logs_after(self.run_id, cursor=after)
-        events = [
-            event
-            for event in events
-            if event.dagster_event_type != DagsterEventType.ASSET_MATERIALIZATION_PLANNED
-        ]
         return [from_event_record(event, self._pipeline_run.pipeline_name) for event in events]
 
     def _get_run_record(self, instance):
@@ -786,25 +791,9 @@ class GrapheneInProgressRunsByStep(graphene.ObjectType):
         name = "InProgressRunsByStep"
 
 
-class GrapheneJobRunsCount(graphene.ObjectType):
-    stepKey = graphene.NonNull(graphene.String)
-    jobNames = non_null_list(graphene.String)
-    count = graphene.NonNull(graphene.Int)
-    sinceLatestMaterialization = graphene.NonNull(graphene.Boolean)
-
-    class Meta:
-        name = "JobRunsCount"
-
-
 class GrapheneLatestRun(graphene.ObjectType):
     stepKey = graphene.NonNull(graphene.String)
     run = graphene.Field(GrapheneRun)
 
     class Meta:
         name = "LatestRun"
-
-
-class GrapheneRunStatsByStep(graphene.Union):
-    class Meta:
-        types = (GrapheneLatestRun, GrapheneJobRunsCount)
-        name = "RunStatsByStep"
